@@ -53,10 +53,10 @@ run_make_cmd(systemtap_session& s, string& make_cmd)
     make_cmd += " >/dev/null";
   else
     make_cmd += " -s >/dev/null 2>&1";
-  
+
   if (s.verbose > 1) clog << "Running " << make_cmd << endl;
   rc = system (make_cmd.c_str());
-  
+
   return rc;
 }
 
@@ -78,8 +78,12 @@ compile_pass (systemtap_session& s)
   if (s.verbose > 3)
     superverbose = "set -x;";
 
+  string redirecterrors = "> /dev/null 2>&1";
+  if (s.verbose > 6)
+    redirecterrors = "";
+
   o << "stap_check_gcc = $(shell " << superverbose << " if $(CC) $(1) -S -o /dev/null -xc /dev/null > /dev/null 2>&1; then echo \"$(1)\"; else echo \"$(2)\"; fi)" << endl;
-  o << "stap_check_build = $(shell " << superverbose << " if $(CC) $(KBUILD_CPPFLAGS) $(CPPFLAGS) $(KBUILD_CFLAGS) $(CFLAGS_KERNEL) $(EXTRA_CFLAGS) $(CFLAGS) -DKBUILD_BASENAME=\\\"" << s.module_name << "\\\" -Werror -S -o /dev/null -xc $(1) > /dev/null 2>&1 ; then echo \"$(2)\"; else echo \"$(3)\"; fi)" << endl;
+  o << "stap_check_build = $(shell " << superverbose << " if $(CC) $(KBUILD_CPPFLAGS) $(CPPFLAGS) $(KBUILD_CFLAGS) $(CFLAGS_KERNEL) $(EXTRA_CFLAGS) $(CFLAGS) -DKBUILD_BASENAME=\\\"" << s.module_name << "\\\" -Werror -S -o /dev/null -xc $(1) " << redirecterrors << " ; then echo \"$(2)\"; else echo \"$(3)\"; fi)" << endl;
 
   o << "SYSTEMTAP_RUNTIME = \"" << s.runtime_path << "\"" << endl;
 
@@ -87,7 +91,16 @@ compile_pass (systemtap_session& s)
 
   string module_cflags = "EXTRA_CFLAGS";
   o << module_cflags << " :=" << endl;
+
+  // XXX: This gruesome hack is needed on some kernels built with separate O=directory,
+  // where files like 2.6.27 x86's asm/mach-*/mach_mpspec.h are not found on the cpp path.
+  // This could be a bug in arch/x86/Makefile that names
+  //      mflags-y += -Iinclude/asm-x86/mach-default
+  // but that path does not exist in an O= build tree.
+  o << module_cflags << " += -Iinclude2/asm/mach-default" << endl;
+
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-hrtimer-rel.c, -DSTAPCONF_HRTIMER_REL,)" << endl;
+  o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-hrtimer-getset-expires.c, -DSTAPCONF_HRTIMER_GETSET_EXPIRES,)" << endl;
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-inode-private.c, -DSTAPCONF_INODE_PRIVATE,)" << endl;
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-constant-tsc.c, -DSTAPCONF_CONSTANT_TSC,)" << endl;
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-tsc-khz.c, -DSTAPCONF_TSC_KHZ,)" << endl;
@@ -95,9 +108,10 @@ compile_pass (systemtap_session& s)
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-x86-uniregs.c, -DSTAPCONF_X86_UNIREGS,)" << endl;
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-nameidata.c, -DSTAPCONF_NAMEIDATA_CLEANUP,)" << endl;
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-unregister-kprobes.c, -DSTAPCONF_UNREGISTER_KPROBES,)" << endl;
-  o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-module-nsections.c, -DSTAPCONF_MODULE_NSECTIONS,)" << endl;
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-real-parent.c, -DSTAPCONF_REAL_PARENT,)" << endl;
   o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-uaccess.c, -DSTAPCONF_LINUX_UACCESS_H,)" << endl;
+  o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-oneachcpu-retry.c, -DSTAPCONF_ONEACHCPU_RETRY,)" << endl;
+  o << module_cflags << " += $(call stap_check_build, $(SYSTEMTAP_RUNTIME)/autoconf-dpath-path.c, -DSTAPCONF_DPATH_PATH,)" << endl;
 #if 0
   /* NB: For now, the performance hit of probe_kernel_read/write (vs. our
    * homegrown safe-access functions) is deemed undesireable, so we'll skip
@@ -120,7 +134,7 @@ compile_pass (systemtap_session& s)
   o << "EXTRA_CFLAGS += -freorder-blocks" << endl; // improve on -Os
 
   // o << "CFLAGS += -fno-unit-at-a-time" << endl;
-    
+
   // Assumes linux 2.6 kbuild
   o << "EXTRA_CFLAGS += -Wno-unused -Werror" << endl;
   o << "EXTRA_CFLAGS += -I\"" << s.runtime_path << "\"" << endl;
@@ -141,7 +155,7 @@ compile_pass (systemtap_session& s)
 	     << strerror(errno) << endl
 	     << "Make sure kernel devel is installed." << endl;
 	return rc;
-    }  
+    }
 
   // Run make
   string make_cmd = string("make")
@@ -149,7 +163,7 @@ compile_pass (systemtap_session& s)
   make_cmd += string(" M=\"") + s.tmpdir + string("\" modules");
 
   rc = run_make_cmd(s, make_cmd);
-  
+
   return rc;
 }
 
@@ -239,12 +253,12 @@ uprobes_pass (systemtap_session& s)
    * stap must fail.
    */
   int rc;
-  if (geteuid() == 0) {
+  if (geteuid() == 0)
     rc = make_uprobes(s);
-    if (rc == 0)
-      rc = copy_uprobes_symbols(s);
-  } else
+  else
     rc = verify_uprobes_uptodate(s);
+  if (rc == 0)
+    rc = copy_uprobes_symbols(s);
   return rc;
 }
 
@@ -261,23 +275,26 @@ run_pass (systemtap_session& s)
     + (s.verbose>1 ? "-v " : "")
     + (s.verbose>2 ? "-v " : "")
     + (s.output_file.empty() ? "" : "-o " + s.output_file + " ");
-  
+
   if (s.cmd != "")
     staprun_cmd += "-c " + cmdstr_quoted(s.cmd) + " ";
-  
+
   if (s.target_pid)
     staprun_cmd += "-t " + stringify(s.target_pid) + " ";
-  
+
   if (s.buffer_size)
     staprun_cmd += "-b " + stringify(s.buffer_size) + " ";
-  
+
   if (s.need_uprobes)
     staprun_cmd += "-u ";
 
+  if (s.load_only)
+    staprun_cmd += "-L ";
+
   staprun_cmd += s.tmpdir + "/" + s.module_name + ".ko";
-  
+
   if (s.verbose>1) clog << "Running " << staprun_cmd << endl;
-  
+
   rc = system (staprun_cmd.c_str ());
   return rc;
 }
