@@ -69,6 +69,8 @@ usage (systemtap_session& s, int exitcode)
     << endl
     << "   or: stap [options] -l PROBE     List matching probes."
     << endl
+    << "   or: stap [options] -L PROBE     List matching probes and local variables."
+    << endl
     << endl
     << "Options:" << endl
     << "   --         end of translator options, script options follow" << endl
@@ -79,7 +81,7 @@ usage (systemtap_session& s, int exitcode)
     << "   -u         unoptimized translation" << (s.unoptimized ? " [set]" : "") << endl
     << "   -w         suppress warnings" << (s.suppress_warnings ? " [set]" : "") << endl
     << "   -g         guru mode" << (s.guru_mode ? " [set]" : "") << endl
-    << "   -P         prologue-searching for function probes" 
+    << "   -P         prologue-searching for function probes"
     << (s.prologue_searching ? " [set]" : "") << endl
     << "   -b         bulk (percpu file) mode" << (s.bulk_mode ? " [set]" : "") << endl
     << "   -s NUM     buffer size in megabytes, instead of "
@@ -106,6 +108,7 @@ usage (systemtap_session& s, int exitcode)
     << "   -c CMD     start the probes, run CMD, and exit when it finishes"
     << endl
     << "   -x PID     sets target() to PID" << endl
+    << "   -F         load module and start probes, then detach" << endl
     << "   -d OBJECT  add unwind/symbol data for OBJECT file";
   if (s.unwindsym_modules.size() == 0)
     clog << endl;
@@ -121,16 +124,15 @@ usage (systemtap_session& s, int exitcode)
 #ifdef HAVE_LIBSQLITE3
     << "   -q         generate information on tapset coverage" << endl
 #endif /* HAVE_LIBSQLITE3 */
+#if 0 /* PR6864: disable temporarily; should merge with -d somehow */
     << "   --kelf     make do with symbol table from vmlinux" << endl
     << "   --kmap[=FILE]" << endl
     << "              make do with symbol table from nm listing" << endl
-    << "   --ignore-vmlinux" << endl
-    << "              for testing, pretend vmlinux can't be found" << endl
-    << "   --ignore-dwarf" << endl
-    << "              for testing, pretend vmlinux and modules lack debug info"
+#endif
+  // Formerly present --ignore-{vmlinux,dwarf} options are for testsuite use
+  // only, and don't belong in the eyesight of a plain user.
     << endl
     ;
-  // -d: dump safety-related external references
 
   exit (exitcode);
 }
@@ -146,6 +148,8 @@ printscript(systemtap_session& s, ostream& o)
 
       for (unsigned i=0; i<s.probes.size(); i++)
         {
+          if (pending_interrupts) return;
+
           derived_probe* p = s.probes[i];
           // NB: p->basest() is not so interesting;
           // p->almost_basest() doesn't quite work, so ...
@@ -165,7 +169,7 @@ printscript(systemtap_session& s, ostream& o)
             {
               assert (a->alias_names.size() >= 1);
               a->alias_names[0]->print(tmps); // XXX: [0] is arbitrary; perhaps print all
-            }            
+            }
           else
             {
               assert (second->locations.size() >= 1);
@@ -178,7 +182,16 @@ printscript(systemtap_session& s, ostream& o)
           // print the alias head name once.
           if (seen.find (pp) == seen.end())
             {
-              o << pp << endl;
+              o << pp;
+              // Print the locals for -L mode only
+              if (s.unoptimized)
+                for (unsigned j=0; j<p->locals.size(); j++)
+                  {
+                    o << " ";
+                    vardecl* v = p->locals[j];
+                    v->printsig (o);
+                  }
+              o << endl;
               seen.insert (pp);
             }
         }
@@ -189,15 +202,17 @@ printscript(systemtap_session& s, ostream& o)
         o << "# global embedded code" << endl;
       for (unsigned i=0; i<s.embeds.size(); i++)
         {
+          if (pending_interrupts) return;
           embeddedcode* ec = s.embeds[i];
           ec->print (o);
           o << endl;
         }
-      
+
       if (s.globals.size() > 0)
         o << "# globals" << endl;
       for (unsigned i=0; i<s.globals.size(); i++)
         {
+          if (pending_interrupts) return;
           vardecl* v = s.globals[i];
           v->printsig (o);
           if (s.verbose && v->init)
@@ -207,12 +222,13 @@ printscript(systemtap_session& s, ostream& o)
             }
           o << endl;
         }
-      
+
       if (s.functions.size() > 0)
         o << "# functions" << endl;
-      for (unsigned i=0; i<s.functions.size(); i++)
+      for (map<string,functiondecl*>::iterator it = s.functions.begin(); it != s.functions.end(); it++)
         {
-          functiondecl* f = s.functions[i];
+          if (pending_interrupts) return;
+          functiondecl* f = it->second;
           f->printsig (o);
           o << endl;
           if (f->locals.size() > 0)
@@ -230,11 +246,12 @@ printscript(systemtap_session& s, ostream& o)
               o << endl;
             }
         }
-      
+
       if (s.probes.size() > 0)
         o << "# probes" << endl;
       for (unsigned i=0; i<s.probes.size(); i++)
         {
+          if (pending_interrupts) return;
           derived_probe* p = s.probes[i];
           p->printsig (o);
           o << endl;
@@ -260,10 +277,16 @@ printscript(systemtap_session& s, ostream& o)
 int pending_interrupts;
 
 extern "C"
-void handle_interrupt (int /* sig */) 
+void handle_interrupt (int sig)
 {
+  if (pending_interrupts == 0)
+    kill (0, sig); // Forward signals to child processes if any.
+
   pending_interrupts ++;
-  if (pending_interrupts > 1) // XXX: should be configurable? time-based?
+  // NB: the "2" below is intended to skip the effect of the self-induced
+  // deferred signal coming from the kill() above.
+
+  if (pending_interrupts > 2) // XXX: should be configurable? time-based?
     {
       char msg[] = "Too many interrupts received, exiting.\n";
       int rc = write (2, msg, sizeof(msg)-1);
@@ -341,9 +364,10 @@ main (int argc, char * const argv [])
   s.consult_symtab = false;
   s.ignore_vmlinux = false;
   s.ignore_dwarf = false;
+  s.load_only = false;
 
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
-  if (s_p != NULL)  
+  if (s_p != NULL)
   {
     s.include_path.push_back (s_p);
   }
@@ -404,7 +428,7 @@ main (int argc, char * const argv [])
         { "ignore-dwarf", 0, &long_opt, LONG_OPT_IGNORE_DWARF },
         { NULL, 0, NULL, 0 }
       };
-      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:d:",
+      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:d:L:F",
                                                           long_options, NULL);
       if (grc < 0)
         break;
@@ -572,6 +596,9 @@ main (int argc, char * const argv [])
           usage (s, 0);
           break;
 
+        case 'L':
+          s.unoptimized = true; // This causes retention of variables for listing_mode
+
         case 'l':
 	  s.suppress_warnings = true;
           s.listing_mode = true;
@@ -586,6 +613,10 @@ main (int argc, char * const argv [])
           have_script = true;
           break;
 
+        case 'F':
+          s.load_only = true;
+	  break;
+
         case 0:
           switch (long_opt)
             {
@@ -690,25 +721,11 @@ main (int argc, char * const argv [])
 
   int rc = 0;
 
-  // override PATH and LC_ALL
-  const char *path = "/bin:/sbin:/usr/bin:/usr/sbin";
-  rc = setenv("PATH", path, 1) || setenv("LC_ALL", "C", 1);
-  if (rc)
-    {
-      const char* e = strerror (errno);
-      cerr << "setenv (\"PATH=" << path << "\" + \"LC_ALL=C\"): "
-           << e << endl;
-    }
-
-  // Get rid of a few standard environment variables (which might
-  // cause us to do unintended things).
-  rc = unsetenv("IFS") || unsetenv("CDPATH") || unsetenv("ENV")
-      || unsetenv("BASH_ENV");
-  if (rc)
-    {
-      const char* e = strerror (errno);
-      cerr << "unsetenv failed: " << e << endl;
-    }
+  // For PR1477, we used to override $PATH and $LC_ALL and other stuff
+  // here.  We seem to use complete pathnames in
+  // buildrun.cxx/tapsets.cxx now, so this is not necessary.  Further,
+  // it interferes with util.cxx:find_executable(), used for $PATH
+  // resolution.
 
   s.kernel_base_release.assign(s.kernel_release, 0, s.kernel_release.find('-'));
 
@@ -727,7 +744,7 @@ main (int argc, char * const argv [])
     const char* tmpdir_env = getenv("TMPDIR");
     if (! tmpdir_env)
       tmpdir_env = "/tmp";
-    
+
     string stapdir = "/stapXXXXXX";
     string tmpdirt = tmpdir_env + stapdir;
     mode_t mask = umask(0);
@@ -750,7 +767,7 @@ main (int argc, char * const argv [])
   // directory.
   s.translated_source = string(s.tmpdir) + "/" + s.module_name + ".c";
 
-  // Set up our handler to catch routine signals, to allow clean 
+  // Set up our handler to catch routine signals, to allow clean
   // and reasonably timely exit.
   setup_signals(&handle_interrupt);
 
@@ -987,6 +1004,7 @@ main (int argc, char * const argv [])
   if (rc || s.last_pass == 3 || pending_interrupts) goto cleanup;
 
   // PASS 4: COMPILATION
+
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
   rc = compile_pass (s);
@@ -1012,7 +1030,7 @@ main (int argc, char * const argv [])
          << endl;
   else
     {
-      // Update cache.
+      // Update cache. Cache cleaning is kicked off at the end of this function.
       if (s.use_cache)
 	add_to_cache(s);
 
