@@ -16,13 +16,16 @@
 #include <iostream>
 #include <vector>
 #include <stdexcept>
+#include <stdint.h>
 
+struct stapfile;
 
 struct source_loc
 {
   std::string file;
   unsigned line;
   unsigned column;
+  stapfile* stap_file;
 };
 
 std::ostream& operator << (std::ostream& o, const source_loc& loc);
@@ -33,7 +36,7 @@ enum parse_context
   };
 
 
-enum token_type 
+enum token_type
   {
     tok_junk, tok_identifier, tok_operator, tok_string, tok_number,
     tok_embedded, tok_keyword
@@ -55,11 +58,11 @@ struct parse_error: public std::runtime_error
 {
   const token* tok;
   bool skip_some;
-  parse_error (const std::string& msg): 
+  parse_error (const std::string& msg):
     runtime_error (msg), tok (0), skip_some (true) {}
-  parse_error (const std::string& msg, const token* t): 
+  parse_error (const std::string& msg, const token* t):
     runtime_error (msg), tok (t), skip_some (true) {}
-  parse_error (const std::string& msg, bool skip): 
+  parse_error (const std::string& msg, bool skip):
     runtime_error (msg), tok (0), skip_some (skip) {}
 };
 
@@ -69,8 +72,10 @@ struct systemtap_session;
 class lexer
 {
 public:
-  token* scan (bool wildcard=false, bool expand_args=true);
+  token* scan (bool wildcard=false);
   lexer (std::istream&, const std::string&, systemtap_session&);
+  std::string get_input_contents ();
+  void set_current_file (stapfile* f);
 
 private:
   int input_get ();
@@ -79,15 +84,15 @@ private:
   int input_peek (unsigned n=0);
   std::istream& input;
   std::string input_name;
-  std::vector<int> lookahead;
+  std::string input_contents;
+  int input_pointer; // index into input_contents
   unsigned cursor_suspend_count;
   unsigned cursor_line;
   unsigned cursor_column;
   systemtap_session& session;
+  stapfile* current_file;
 };
 
-
-struct stapfile;
 struct probe;
 struct probe_alias;
 struct vardecl;
@@ -132,7 +137,7 @@ private:
 
   // preprocessing subordinate
   std::vector<const token*> enqueued_pp;
-  const token* scan_pp (bool wildcard=false, bool expand_args=true);
+  const token* scan_pp (bool wildcard=false);
 
   // scanning state
   const token* last ();
@@ -141,7 +146,7 @@ private:
 
   const token* last_t; // the last value returned by peek() or next()
   const token* next_t; // lookahead token
-  
+
   // expectations
   const token* expect_known (token_type tt, std::string const & expected);
   const token* expect_unknown (token_type tt, std::string & target);
