@@ -34,17 +34,19 @@ void EXPORT_FN(stp_print_flush) (_stp_pbuf *pb)
 #ifdef STP_BULKMODE
 	{
 #ifdef NO_PERCPU_HEADERS
-		void *buf = relay_reserve(_stp_utt->rchan, len);		
+		void *buf = utt_reserve(_stp_utt, len);
 		if (likely(buf))
 			memcpy(buf, pb->buf, len);
 		else
 			atomic_inc (&_stp_transport_failures);
 #else
-		struct _stp_trace *t = relay_reserve(_stp_utt->rchan, sizeof(*t) + len);
-		if (likely(t)) {
-			t->sequence = _stp_seq_inc();
-			t->pdu_len = len;
-			memcpy((void *) t + sizeof(*t), pb->buf, len);
+		void *buf = utt_reserve(_stp_utt,
+					sizeof(struct _stp_trace) + len);
+		if (likely(buf)) {
+			struct _stp_trace t = {	.sequence = _stp_seq_inc(),
+						.pdu_len = len};
+			memcpy(buf, &t, sizeof(t)); // prevent unaligned access
+			memcpy(buf + sizeof(t), pb->buf, len);
 		} else 
 			atomic_inc (&_stp_transport_failures);
 #endif
@@ -54,7 +56,7 @@ void EXPORT_FN(stp_print_flush) (_stp_pbuf *pb)
 		void *buf;
 		unsigned long flags;
 		spin_lock_irqsave(&_stp_print_lock, flags);
-		buf = relay_reserve(_stp_utt->rchan, len);		
+		buf = utt_reserve(_stp_utt, len);
 		if (likely(buf))
 			memcpy(buf, pb->buf, len);
 		else
