@@ -58,29 +58,39 @@ static struct
 
 #define _stp_seq_inc() (atomic_inc_return(&_stp_seq.seq))
 
-/* TEST_MODE is always defined by systemtap */
-#ifdef TEST_MODE
-#define SYSTEMTAP 1
-#else
-#define MAXTRYLOCK 1000
-#define TRYLOCKDELAY 100
-#endif
-
 #ifndef MAXSTRINGLEN
 #define MAXSTRINGLEN 128
 #endif
 
+#ifndef MAXTRACE
+#define MAXTRACE 20
+#endif
+
+#ifdef CONFIG_FRAME_POINTER
+/* Just because frame pointers are available does not mean we can trust them. */
+#ifndef STP_USE_DWARF_UNWINDER
+#define STP_USE_FRAME_POINTER
+#endif
+#endif
+
+/* dwarf unwinder only tested so far on i386 and x86_64,
+   but globally disabled for now */
+#if 0
+// !defined(STP_USE_FRAME_BUFFER) && (defined(__i386__) || defined(__x86_64__))
+#define STP_USE_DWARF_UNWINDER
+#endif
+
 #include "alloc.c"
 #include "print.c"
 #include "string.c"
 #include "io.c"
 #include "arith.c"
 #include "copy.c"
-#include "sym.h"
 #include "sym.c"
 #ifdef STP_PERFMON
 #include "perf.c"
 #endif
+#include "addr-map.c"
 
 /* Support functions for int64_t module parameters. */
 int param_set_int64_t(const char *val, struct kernel_param *kp)
