@@ -10,27 +10,34 @@
 #ifndef _STP_SYM_H_
 #define _STP_SYM_H_
 
-#define STP_MODULE_NAME_LEN 64
-
 struct _stp_symbol {
 	unsigned long addr;
 	const char *symbol;
 };
 
+struct _stp_section {
+        const char *name;
+        unsigned long addr; /* XXX: belongs in per-address-space tables */
+	struct _stp_symbol *symbols;  /* ordered by address */
+  	unsigned num_symbols;
+};
+
+
 struct _stp_module {
-	/* the module name, or "" for kernel */
-	char name[STP_MODULE_NAME_LEN];
-	
+        const char* name;
+	struct _stp_section *sections;
+  	unsigned num_sections;
+
 	/* A pointer to the struct module. Note that we cannot */
 	/* trust this because as of 2.6.19, there are not yet */
 	/* any notifier hooks that will tell us when a module */
 	/* is unloading. */
   	unsigned long module; /* XXX: why not struct module * ? */
-	
-	struct _stp_symbol *sections;
-  	unsigned num_sections;
-	struct _stp_symbol *symbols;  /* ordered by address */
-  	unsigned num_symbols;
+
+	 /* This is to undo .debug_frame relocation performed by elfutils, */
+	 /* which is done during the translate phase when we encode the    */
+	 /* unwind data into the module. See adjustStartLoc() in unwind.c. */
+  	unsigned long dwarf_module_base;
 
 	/* the stack unwind data for this module */
 	void *unwind_data;
@@ -38,23 +45,24 @@ struct _stp_module {
 	uint32_t unwind_data_len;
 	uint32_t unwind_hdr_len;
 	uint32_t unwind_is_ehframe; /* unwind data comes from .eh_frame */
+	/* build-id information */
+	unsigned char *build_id_bits;
+	unsigned long  build_id_offset;
+	unsigned long  notes_sect;
+	int build_id_len;
 };
 
 
 /* Defined by translator-generated stap-symbols.h. */
 struct _stp_module *_stp_modules [];
-int _stp_num_modules;
+unsigned _stp_num_modules;
 
 
-#if 0
-/* the array of modules ordered by addresses */
-struct _stp_module *_stp_modules_by_addr[STP_MAX_MODULES];
-#endif
-
 /* the number of modules in the arrays */
 
 static unsigned long _stp_kretprobe_trampoline = 0;
 
 unsigned long _stp_module_relocate (const char *module, const char *section, unsigned long offset);
 static struct _stp_module *_stp_get_unwind_info (unsigned long addr);
+
 #endif /* _STP_SYM_H_ */
