@@ -1,6 +1,17 @@
+#ifndef TASK_FINDER_C
+#define TASK_FINDER_C
+
+#if ! defined(CONFIG_UTRACE)
+#error "Need CONFIG_UTRACE!"
+#endif
+
+#include <linux/utrace.h>
 #include <linux/list.h>
 #include <linux/binfmts.h>
+#include <linux/mount.h>
+
 #include "syscall.h"
+#include "utrace_compatibility.h"
 #include "task_finder_vma.c"
 
 static LIST_HEAD(__stp_task_finder_list);
@@ -12,6 +23,10 @@ struct stap_task_finder_target;
 #define __STP_TF_STOPPING	2
 #define __STP_TF_STOPPED	3
 atomic_t __stp_task_finder_state = ATOMIC_INIT(__STP_TF_STARTING);
+atomic_t __stp_inuse_count = ATOMIC_INIT (0);
+
+#define __stp_tf_handler_start() (atomic_inc(&__stp_inuse_count))
+#define __stp_tf_handler_end() (atomic_dec(&__stp_inuse_count))
 
 #ifdef DEBUG_TASK_FINDER
 atomic_t __stp_attach_count = ATOMIC_INIT (0);
@@ -19,26 +34,30 @@ atomic_t __stp_attach_count = ATOMIC_INIT (0);
 #define debug_task_finder_attach() (atomic_inc(&__stp_attach_count))
 #define debug_task_finder_detach() (atomic_dec(&__stp_attach_count))
 #define debug_task_finder_report() (_stp_dbug(__FUNCTION__, __LINE__, \
-	"attach count: %d\n", atomic_read(&__stp_attach_count)))
+					      "attach count: %d, inuse count: %d\n", \
+					      atomic_read(&__stp_attach_count), \
+					      atomic_read(&__stp_inuse_count)))
 #else
 #define debug_task_finder_attach()	/* empty */
 #define debug_task_finder_detach()	/* empty */
 #define debug_task_finder_report()	/* empty */
 #endif
 
-typedef int (*stap_task_finder_callback)(struct task_struct *tsk,
+typedef int (*stap_task_finder_callback)(struct stap_task_finder_target *tgt,
+					 struct task_struct *tsk,
 					 int register_p,
-					 int process_p,
-					 struct stap_task_finder_target *tgt);
+					 int process_p);
 
-typedef int (*stap_task_finder_vm_callback)(struct task_struct *tsk,
+typedef int (*stap_task_finder_vm_callback)(struct stap_task_finder_target *tgt,
+					    struct task_struct *tsk,
 					    int map_p, char *vm_path,
 					    unsigned long vm_start,
 					    unsigned long vm_end,
 					    unsigned long vm_pgoff);
 
 #ifdef DEBUG_TASK_FINDER_VMA
-int __stp_tf_vm_cb(struct task_struct *tsk,
+int __stp_tf_vm_cb(struct stap_task_finder_target *tgt,
+		   struct task_struct *tsk,
 		   int map_p, char *vm_path,
 		   unsigned long vm_start,
 		   unsigned long vm_end,
@@ -47,6 +66,15 @@ int __stp_tf_vm_cb(struct task_struct *tsk,
 	_stp_dbug(__FUNCTION__, __LINE__,
 		  "vm_cb: tsk %d:%d path %s, start 0x%08lx, end 0x%08lx, offset 0x%lx\n",
 		  tsk->pid, map_p, vm_path, vm_start, vm_end, vm_pgoff);
+	if (map_p) {
+		// FIXME: What should we do with vm_path?  We can't save
+		// the vm_path pointer itself, but we don't have any
+		// storage space allocated to save it in...
+		stap_add_vma_map_info(tsk, vm_start, vm_end, vm_pgoff);
+	}
+	else {
+		stap_remove_vma_map_info(tsk, vm_start, vm_end, vm_pgoff);
+	}
 	return 0;
 }
 #endif
@@ -61,36 +89,67 @@ struct stap_task_finder_target {
 	size_t pathlen;
 
 /* public: */
-    	const char *pathname;
+	const char *pathname;
 	pid_t pid;
 	stap_task_finder_callback callback;
 	stap_task_finder_vm_callback vm_callback;
 };
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_death(struct utrace_attached_engine *engine,
 				      struct task_struct *tsk);
+#else
+static u32
+__stp_utrace_task_finder_target_death(struct utrace_attached_engine *engine,
+				      struct task_struct *tsk,
+				      bool group_dead, int signal);
+#endif
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_quiesce(struct utrace_attached_engine *engine,
 					struct task_struct *tsk);
+#else
+static u32
+__stp_utrace_task_finder_target_quiesce(enum utrace_resume_action action,
+					struct utrace_attached_engine *engine,
+					struct task_struct *tsk,
+					unsigned long event);
+#endif
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_syscall_entry(struct utrace_attached_engine *engine,
 					      struct task_struct *tsk,
 					      struct pt_regs *regs);
+#else
+static u32
+__stp_utrace_task_finder_target_syscall_entry(enum utrace_resume_action action,
+					      struct utrace_attached_engine *engine,
+					      struct task_struct *tsk,
+					      struct pt_regs *regs);
+#endif
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engine,
 					     struct task_struct *tsk,
 					     struct pt_regs *regs);
+#else
+static u32
+__stp_utrace_task_finder_target_syscall_exit(enum utrace_resume_action action,
+					     struct utrace_attached_engine *engine,
+					     struct task_struct *tsk,
+					     struct pt_regs *regs);
+#endif
 
 static int
 stap_register_task_finder_target(struct stap_task_finder_target *new_tgt)
 {
 	// Since this __stp_task_finder_list is (currently) only
-        // written to in one big setup operation before the task
-        // finder process is started, we don't need to lock it.
+	// written to in one big setup operation before the task
+	// finder process is started, we don't need to lock it.
 	struct list_head *node;
 	struct stap_task_finder_target *tgt = NULL;
 	int found_node = 0;
@@ -121,8 +180,10 @@ stap_register_task_finder_target(struct stap_task_finder_target *new_tgt)
 		    && ((new_tgt->pathlen > 0
 			 && tgt->pathlen == new_tgt->pathlen
 			 && strcmp(tgt->pathname, new_tgt->pathname) == 0)
-			/* pid-based target */
-			|| (new_tgt->pid != 0 && tgt->pid == new_tgt->pid))) {
+			/* pid-based target (a specific pid or all
+			 * pids) */
+			|| (new_tgt->pathlen == 0
+			    && tgt->pid == new_tgt->pid))) {
 			found_node = 1;
 			break;
 		}
@@ -141,12 +202,73 @@ stap_register_task_finder_target(struct stap_task_finder_target *new_tgt)
 	return 0;
 }
 
+static int
+stap_utrace_detach(struct task_struct *tsk,
+		   const struct utrace_engine_ops *ops)
+{
+	struct utrace_attached_engine *engine;
+	struct mm_struct *mm;
+	int rc = 0;
+
+	// Ignore init
+	if (tsk == NULL || tsk->pid <= 1)
+		return 0;
+
+	// Notice we're not calling get_task_mm() here.  Normally we
+	// avoid tasks with no mm, because those are kernel threads.
+	// So, why is this function different?  When a thread is in
+	// the process of dying, its mm gets freed.  Then, later the
+	// thread gets in the dying state and the thread's DEATH event
+	// handler gets called (if any).
+	//
+	// If a thread is in this "mortally wounded" state - no mm
+	// but not dead - and at that moment this function is called,
+	// we'd miss detaching from it if we were checking to see if
+	// it had an mm.
+
+	engine = utrace_attach_task(tsk, UTRACE_ATTACH_MATCH_OPS, ops, 0);
+	if (IS_ERR(engine)) {
+		rc = -PTR_ERR(engine);
+		if (rc != ENOENT) {
+			_stp_error("utrace_attach_task returned error %d on pid %d",
+				   rc, tsk->pid);
+		}
+		else {
+			rc = 0;
+		}
+	}
+	else if (unlikely(engine == NULL)) {
+		_stp_error("utrace_attach returned NULL on pid %d",
+			   (int)tsk->pid);
+		rc = EFAULT;
+	}
+	else {
+		rc = utrace_control(tsk, engine, UTRACE_DETACH);
+		switch (rc) {
+		case 0:			/* success */
+			debug_task_finder_detach();
+			break;
+		case -ESRCH:	    /* REAP callback already begun */
+		case -EALREADY:	    /* DEATH callback already begun */
+			rc = 0;	    /* ignore these errors*/
+			break;
+		default:
+			rc = -rc;
+			_stp_error("utrace_detach returned error %d on pid %d",
+				   rc, tsk->pid);
+			break;
+		}
+		utrace_engine_put(engine);
+	}
+	return rc;
+}
+
 static void
 stap_utrace_detach_ops(struct utrace_engine_ops *ops)
 {
 	struct task_struct *grp, *tsk;
 	struct utrace_attached_engine *engine;
-	long error = 0;
+	int rc = 0;
 	pid_t pid = 0;
 
 	// Notice we're not calling get_task_mm() in this loop. In
@@ -164,31 +286,12 @@ stap_utrace_detach_ops(struct utrace_engine_ops *ops)
 
 	rcu_read_lock();
 	do_each_thread(grp, tsk) {
-		if (tsk == NULL || tsk->pid <= 1)
-			continue;
-
-		engine = utrace_attach(tsk, UTRACE_ATTACH_MATCH_OPS,
-				       ops, 0);
-		if (IS_ERR(engine)) {
-			error = -PTR_ERR(engine);
-			if (error != ENOENT) {
-				pid = tsk->pid;
-				goto udo_err;
-			}
-			error = 0;
-		}
-		else if (engine != NULL) {
-			utrace_detach(tsk, engine);
-			debug_task_finder_detach();
-		}
+		rc = stap_utrace_detach(tsk, ops);
+		if (rc != 0)
+			goto udo_err;
 	} while_each_thread(grp, tsk);
 udo_err:
 	rcu_read_unlock();
-
-	if (error != 0) {
-		_stp_error("utrace_attach returned error %d on pid %d",
-			   error, pid);
-	}
 	debug_task_finder_report();
 }
 
@@ -240,11 +343,11 @@ __stp_get_mm_path(struct mm_struct *mm, char *buf, int buflen)
 		vma = vma->vm_next;
 	}
 	if (vma) {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,25)
+#ifdef STAPCONF_DPATH_PATH
+		rc = d_path(&(vma->vm_file->f_path), buf, buflen);
+#else
 		rc = d_path(vma->vm_file->f_dentry, vma->vm_file->f_vfsmnt,
 			    buf, buflen);
-#else
-		rc = d_path(&(vma->vm_file->f_path), buf, buflen);
 #endif
 	}
 	else {
@@ -255,28 +358,46 @@ __stp_get_mm_path(struct mm_struct *mm, char *buf, int buflen)
 	return rc;
 }
 
+/*
+ * All user threads get an engine with __STP_TASK_FINDER_EVENTS events
+ * attached to it so the task_finder layer can monitor new thread
+ * creation/death.
+ */
 #define __STP_TASK_FINDER_EVENTS (UTRACE_EVENT(CLONE)		\
 				  | UTRACE_EVENT(EXEC)		\
 				  | UTRACE_EVENT(DEATH))
 
-#define __STP_ATTACHED_TASK_BASE_EVENTS (UTRACE_EVENT(DEATH))
+/*
+ * __STP_TASK_BASE_EVENTS: base events for stap_task_finder_target's
+ * without a vm_callback
+ *
+ * __STP_TASK_VM_BASE_EVENTS: base events for
+ * stap_task_finder_target's with a vm_callback
+ */
+#define __STP_TASK_BASE_EVENTS	(UTRACE_EVENT(DEATH))
 
-#define __STP_ATTACHED_TASK_VM_BASE_EVENTS (__STP_ATTACHED_TASK_BASE_EVENTS \
-					    | UTRACE_EVENT(SYSCALL_ENTRY) \
-					    | UTRACE_EVENT(SYSCALL_EXIT))
+#define __STP_TASK_VM_BASE_EVENTS (__STP_TASK_BASE_EVENTS	\
+				   | UTRACE_EVENT(SYSCALL_ENTRY)\
+				   | UTRACE_EVENT(SYSCALL_EXIT))
 
-#define __STP_ATTACHED_TASK_VM_EVENTS (__STP_ATTACHED_TASK_VM_BASE_EVENTS \
-				       | UTRACE_ACTION_QUIESCE	\
-				       | UTRACE_EVENT(QUIESCE))
+/*
+ * All "interesting" threads get an engine with
+ * __STP_ATTACHED_TASK_EVENTS events attached to it.  After the thread
+ * quiesces, we reset the events to __STP_ATTACHED_TASK_BASE_EVENTS
+ * events.
+ */
+#define __STP_ATTACHED_TASK_EVENTS (__STP_TASK_BASE_EVENTS	\
+				    | UTRACE_EVENT(QUIESCE))
 
-#define __STP_ATTACHED_TASK_EVENTS(tgt) \
-	((((tgt)->vm_callback) == NULL) ? __STP_ATTACHED_TASK_BASE_EVENTS \
-	 : __STP_ATTACHED_TASK_VM_EVENTS)
+#define __STP_ATTACHED_TASK_BASE_EVENTS(tgt) \
+	((((tgt)->vm_callback) == NULL) ? __STP_TASK_BASE_EVENTS \
+	 : __STP_TASK_VM_BASE_EVENTS)
 
 static int
-stap_utrace_attach(struct task_struct *tsk,
-		   const struct utrace_engine_ops *ops, void *data,
-		   unsigned long event_flags)
+__stp_utrace_attach(struct task_struct *tsk,
+		    const struct utrace_engine_ops *ops, void *data,
+		    unsigned long event_flags,
+		    enum utrace_resume_action action)
 {
 	struct utrace_attached_engine *engine;
 	struct mm_struct *mm;
@@ -292,7 +413,7 @@ stap_utrace_attach(struct task_struct *tsk,
 		return EPERM;
 	mmput(mm);
 
-	engine = utrace_attach(tsk, UTRACE_ATTACH_CREATE, ops, data);
+	engine = utrace_attach_task(tsk, UTRACE_ATTACH_CREATE, ops, data);
 	if (IS_ERR(engine)) {
 		int error = -PTR_ERR(engine);
 		if (error != ENOENT) {
@@ -307,16 +428,52 @@ stap_utrace_attach(struct task_struct *tsk,
 		rc = EFAULT;
 	}
 	else {
-		rc = utrace_set_flags(tsk, engine, event_flags);
-		if (rc == 0)
+		rc = utrace_set_events(tsk, engine, event_flags);
+		if (rc == -EINPROGRESS) {
+			/*
+			 * It's running our callback, so we have to
+			 * synchronize.  We can't keep rcu_read_lock,
+			 * so the task pointer might die.  But it's
+			 * safe to call utrace_barrier() even with a
+			 * stale task pointer, if we have an engine
+			 * ref.
+			 */
+			rc = utrace_barrier(tsk, engine);
+			if (rc != 0)
+				_stp_error("utrace_barrier returned error %d on pid %d",
+					   rc, (int)tsk->pid);
+		}
+		if (rc == 0) {
 			debug_task_finder_attach();
+
+			if (action != UTRACE_RESUME) {
+				rc = utrace_control(tsk, engine, UTRACE_STOP);
+				/* EINPROGRESS means we must wait for
+				 * a callback, which is what we want. */
+				if (rc != 0 && rc != -EINPROGRESS)
+					_stp_error("utrace_control returned error %d on pid %d",
+						   rc, (int)tsk->pid);
+				else
+					rc = 0;
+			}
+
+		}
 		else
-			_stp_error("utrace_set_flags returned error %d on pid %d",
+			_stp_error("utrace_set_events2 returned error %d on pid %d",
 				   rc, (int)tsk->pid);
+		utrace_engine_put(engine);
 	}
 	return rc;
 }
 
+static int
+stap_utrace_attach(struct task_struct *tsk,
+		   const struct utrace_engine_ops *ops, void *data,
+		   unsigned long event_flags)
+{
+	return __stp_utrace_attach(tsk, ops, data, event_flags, UTRACE_RESUME);
+}
+
 static inline void
 __stp_utrace_attach_match_filename(struct task_struct *tsk,
 				   const char * const filename,
@@ -325,23 +482,29 @@ __stp_utrace_attach_match_filename(struct task_struct *tsk,
 	size_t filelen;
 	struct list_head *tgt_node;
 	struct stap_task_finder_target *tgt;
-	int found_node = 0;
 
 	filelen = strlen(filename);
 	list_for_each(tgt_node, &__stp_task_finder_list) {
+		struct list_head *cb_node;
+
 		tgt = list_entry(tgt_node, struct stap_task_finder_target,
 				 list);
-		// Note that we don't bother with looking for pids
-		// here, since they are handled at startup.
-		if (tgt != NULL && tgt->pathlen > 0
-		    && tgt->pathlen == filelen
-		    && strcmp(tgt->pathname, filename) == 0) {
-			found_node = 1;
-			break;
-		}
-	}
-	if (found_node) {
-		struct list_head *cb_node;
+		// If we've got a matching pathname or we're probing
+		// all threads, we've got a match.  We've got to keep
+		// matching since a single thread could match a
+		// pathname and match an "all thread" probe.
+		if (tgt == NULL)
+			continue;
+		else if (tgt->pathlen > 0
+			 && (tgt->pathlen != filelen
+			     || strcmp(tgt->pathname, filename) != 0))
+			continue;
+		/* Ignore pid-based target, they were handled at startup. */
+		else if (tgt->pid != 0)
+			continue;
+		/* Notice that "pid == 0" (which means to probe all
+		 * threads) falls through. */
+
 		list_for_each(cb_node, &tgt->callback_list_head) {
 			struct stap_task_finder_target *cb_tgt;
 			int rc;
@@ -352,34 +515,41 @@ __stp_utrace_attach_match_filename(struct task_struct *tsk,
 			if (cb_tgt == NULL)
 				continue;
 
-			if (cb_tgt->callback != NULL) {
-				int rc = cb_tgt->callback(tsk, register_p,
-							  process_p, cb_tgt);
-				if (rc != 0) {
-					_stp_error("callback for %d failed: %d",
-						   (int)tsk->pid, rc);
-					break;
-				}
-			}
-
 			// Set up events we need for attached tasks.
+			// When register_p is set, we won't actually
+			// call the callback here - we'll call it when
+			// the thread gets quiesced.  When register_p
+			// isn't set, we can go ahead and call the
+			// callback.
 			if (register_p) {
-				rc = stap_utrace_attach(tsk, &cb_tgt->ops,
-							cb_tgt,
-							__STP_ATTACHED_TASK_EVENTS(cb_tgt));
+				rc = __stp_utrace_attach(tsk, &cb_tgt->ops,
+							 cb_tgt,
+							 __STP_ATTACHED_TASK_EVENTS,
+							 UTRACE_STOP);
 				if (rc != 0 && rc != EPERM)
 					break;
 				cb_tgt->engine_attached = 1;
 			}
 			else {
-				struct utrace_attached_engine *engine;
-				engine = utrace_attach(tsk,
-						       UTRACE_ATTACH_MATCH_OPS,
-						       &cb_tgt->ops, 0);
-				if (! IS_ERR(engine) && engine != NULL) {
-					utrace_detach(tsk, engine);
-					debug_task_finder_detach();
+				if (cb_tgt->callback != NULL) {
+					rc = cb_tgt->callback(cb_tgt, tsk,
+							      register_p,
+							      process_p);
+					if (rc != 0) {
+						_stp_error("callback for %d failed: %d",
+							   (int)tsk->pid, rc);
+						break;
+					}
 				}
+
+				rc = stap_utrace_detach(tsk, &cb_tgt->ops);
+				if (rc != 0)
+					break;
+
+				// Note that we don't want to set
+				// engine_attached to 0 here - only
+				// when *all* threads using this
+				// engine have been detached.
 			}
 		}
 	}
@@ -441,44 +611,74 @@ __stp_utrace_attach_match_tsk(struct task_struct *path_tsk,
 	return;
 }
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_report_clone(struct utrace_attached_engine *engine,
 				      struct task_struct *parent,
 				      unsigned long clone_flags,
 				      struct task_struct *child)
+#else
+static u32
+__stp_utrace_task_finder_report_clone(enum utrace_resume_action action,
+				      struct utrace_attached_engine *engine,
+				      struct task_struct *parent,
+				      unsigned long clone_flags,
+				      struct task_struct *child)
+#endif
 {
 	int rc;
 	struct mm_struct *mm;
 	char *mmpath_buf;
 	char *mmpath;
 
-	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING)
-		return UTRACE_ACTION_RESUME;
+	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
+		debug_task_finder_detach();
+		return UTRACE_DETACH;
+	}
+
+	__stp_tf_handler_start();
 
 	// On clone, attach to the child.
-	rc = stap_utrace_attach(child, engine->ops, 0,
-				__STP_TASK_FINDER_EVENTS);
-	if (rc != 0 && rc != EPERM)
-		return UTRACE_ACTION_RESUME;
+	rc = __stp_utrace_attach(child, engine->ops, 0,
+				 __STP_TASK_FINDER_EVENTS, UTRACE_RESUME);
+	if (rc != 0 && rc != EPERM) {
+		__stp_tf_handler_end();
+		return UTRACE_RESUME;
+	}
 
 	__stp_utrace_attach_match_tsk(parent, child, 1,
 				      (clone_flags & CLONE_THREAD) == 0);
-	return UTRACE_ACTION_RESUME;
+	__stp_tf_handler_end();
+	return UTRACE_RESUME;
 }
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_report_exec(struct utrace_attached_engine *engine,
 				     struct task_struct *tsk,
 				     const struct linux_binprm *bprm,
 				     struct pt_regs *regs)
+#else
+static u32
+__stp_utrace_task_finder_report_exec(enum utrace_resume_action action,
+				     struct utrace_attached_engine *engine,
+				     struct task_struct *tsk,
+				     const struct linux_binfmt *fmt,
+				     const struct linux_binprm *bprm,
+				     struct pt_regs *regs)
+#endif
 {
 	size_t filelen;
 	struct list_head *tgt_node;
 	struct stap_task_finder_target *tgt;
 	int found_node = 0;
 
-	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING)
-		return UTRACE_ACTION_RESUME;
+	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
+		debug_task_finder_detach();
+		return UTRACE_DETACH;
+	}
+
+	__stp_tf_handler_start();
 
 	// When exec'ing, we need to let callers detach from the
 	// parent thread (if necessary).  For instance, assume
@@ -501,28 +701,44 @@ __stp_utrace_task_finder_report_exec(struct utrace_attached_engine *engine,
 	// relative.
 	__stp_utrace_attach_match_tsk(tsk, tsk, 1, 1);
 
-	return UTRACE_ACTION_RESUME;
+	__stp_tf_handler_end();
+	return UTRACE_RESUME;
 }
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 stap_utrace_task_finder_report_death(struct utrace_attached_engine *engine,
 				     struct task_struct *tsk)
+#else
+static u32
+stap_utrace_task_finder_report_death(struct utrace_attached_engine *engine,
+				     struct task_struct *tsk,
+				     bool group_dead, int signal)
+#endif
 {
 	debug_task_finder_detach();
-	return UTRACE_ACTION_DETACH;
+	return UTRACE_DETACH;
 }
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_death(struct utrace_attached_engine *engine,
 				      struct task_struct *tsk)
+#else
+static u32
+__stp_utrace_task_finder_target_death(struct utrace_attached_engine *engine,
+				      struct task_struct *tsk,
+				      bool group_dead, int signal)
+#endif
 {
 	struct stap_task_finder_target *tgt = engine->data;
 
 	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
 		debug_task_finder_detach();
-		return UTRACE_ACTION_DETACH;
+		return UTRACE_DETACH;
 	}
 
+	__stp_tf_handler_start();
 	// The first implementation of this added a
 	// UTRACE_EVENT(DEATH) handler to
 	// __stp_utrace_task_finder_ops.  However, dead threads don't
@@ -536,33 +752,80 @@ __stp_utrace_task_finder_target_death(struct utrace_attached_engine *engine,
 		int rc;
 
 		// Call the callback
-		rc = tgt->callback(tsk, 0,
-				   (atomic_read(&tsk->signal->live) == 0),
-				   tgt);
+		rc = tgt->callback(tgt, tsk, 0,
+				   ((tsk->signal == NULL)
+				    || (atomic_read(&tsk->signal->live) == 0)));
 		if (rc != 0) {
 			_stp_error("death callback for %d failed: %d",
 				   (int)tsk->pid, rc);
 		}
 	}
+	__stp_tf_handler_end();
 	debug_task_finder_detach();
-	return UTRACE_ACTION_DETACH;
+	return UTRACE_DETACH;
 }
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_quiesce(struct utrace_attached_engine *engine,
 					struct task_struct *tsk)
+#else
+static u32
+__stp_utrace_task_finder_target_quiesce(enum utrace_resume_action action,
+					struct utrace_attached_engine *engine,
+					struct task_struct *tsk,
+					unsigned long event)
+#endif
 {
 	struct stap_task_finder_target *tgt = engine->data;
-
-	// Turn off quiesce handling.
-	utrace_set_flags(tsk, engine, __STP_ATTACHED_TASK_VM_BASE_EVENTS);
+	int rc;
 
 	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
 		debug_task_finder_detach();
-		return UTRACE_ACTION_DETACH;
+		return UTRACE_DETACH;
 	}
 
-	if (tgt != NULL && tgt->vm_callback != NULL) {
+	if (tgt == NULL) {
+		debug_task_finder_detach();
+		return UTRACE_DETACH;
+	}
+
+	__stp_tf_handler_start();
+
+	// Turn off quiesce handling
+	rc = utrace_set_events(tsk, engine,
+			       __STP_ATTACHED_TASK_BASE_EVENTS(tgt));
+
+	if (rc == -EINPROGRESS) {
+		/*
+		 * It's running our callback, so we have to
+		 * synchronize.  We can't keep rcu_read_lock,
+		 * so the task pointer might die.  But it's
+		 * safe to call utrace_barrier() even with
+		 * a stale task pointer, if we have an engine ref.
+		 */
+		rc = utrace_barrier(tsk, engine);
+		if (rc != 0)
+			_stp_error("utrace_barrier returned error %d on pid %d",
+				   rc, (int)tsk->pid);
+		rc = utrace_set_events(tsk, engine,
+				       __STP_ATTACHED_TASK_BASE_EVENTS(tgt));
+	}
+	if (rc != 0)
+		_stp_error("utrace_set_events returned error %d on pid %d",
+			   rc, (int)tsk->pid);
+
+	if (tgt->callback != NULL) {
+		/* Call the callback.  Assume that if the thread is a
+		 * thread group leader, it is a process. */
+		rc = tgt->callback(tgt, tsk, 1, (tsk->pid == tsk->tgid));
+		if (rc != 0) {
+			_stp_error("callback for %d failed: %d",
+				   (int)tsk->pid, rc);
+		}
+	}
+
+	if (tgt->vm_callback != NULL) {
 		struct mm_struct *mm;
 		char *mmpath_buf;
 		char *mmpath;
@@ -587,17 +850,18 @@ __stp_utrace_task_finder_target_quiesce(struct utrace_attached_engine *engine,
 		vma = mm->mmap;
 		while (vma) {
 			if (vma->vm_file) {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,25)
-				mmpath = d_path(vma->vm_file->f_dentry,
-						vma->vm_file->f_vfsmnt,
+#ifdef STAPCONF_DPATH_PATH
+				mmpath = d_path(&(vma->vm_file->f_path),
 						mmpath_buf, PATH_MAX);
 #else
-				mmpath = d_path(&(vma->vm_file->f_path),
+				mmpath = d_path(vma->vm_file->f_dentry,
+						vma->vm_file->f_vfsmnt,
 						mmpath_buf, PATH_MAX);
 #endif
 				if (mmpath) {
 					// Call the callback
-					rc = tgt->vm_callback(tsk, 1, mmpath,
+					rc = tgt->vm_callback(tgt, tsk, 1,
+							      mmpath,
 							      vma->vm_start,
 							      vma->vm_end,
 							      (vma->vm_pgoff
@@ -621,7 +885,8 @@ __stp_utrace_task_finder_target_quiesce(struct utrace_attached_engine *engine,
 	}
 
 utftq_out:
-	return (UTRACE_ACTION_NEWSTATE | UTRACE_ACTION_RESUME);
+	__stp_tf_handler_end();
+	return UTRACE_RESUME;
 }
 
 
@@ -638,10 +903,18 @@ __stp_find_file_based_vma(struct mm_struct *mm, unsigned long addr)
 	return vma;
 }
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_syscall_entry(struct utrace_attached_engine *engine,
 					      struct task_struct *tsk,
 					      struct pt_regs *regs)
+#else
+static u32
+__stp_utrace_task_finder_target_syscall_entry(enum utrace_resume_action action,
+					      struct utrace_attached_engine *engine,
+					      struct task_struct *tsk,
+					      struct pt_regs *regs)
+#endif
 {
 	struct stap_task_finder_target *tgt = engine->data;
 	unsigned long syscall_no;
@@ -649,24 +922,29 @@ __stp_utrace_task_finder_target_syscall_entry(struct utrace_attached_engine *eng
 	struct vm_area_struct *vma;
 	unsigned long *arg0_addr, arg0;
 	int rc;
+#if defined(__ia64__)
+	struct { unsigned long *unwaddr; } _c = {.unwaddr = NULL}, *c = &_c;
+#endif
 
 	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
 		debug_task_finder_detach();
-		return UTRACE_ACTION_DETACH;
+		return UTRACE_DETACH;
 	}
 
 	if (tgt == NULL || tgt->vm_callback == NULL)
-		return UTRACE_ACTION_RESUME;
+		return UTRACE_RESUME;
 
 	// See if syscall is one we're interested in.
 	//
 	// FIXME: do we need to handle mremap()?
 	syscall_no = __stp_user_syscall_nr(regs);
 	if (syscall_no != MMAP_SYSCALL_NO(tsk)
+	    && syscall_no != MMAP2_SYSCALL_NO(tsk)
 	    && syscall_no != MPROTECT_SYSCALL_NO(tsk)
 	    && syscall_no != MUNMAP_SYSCALL_NO(tsk))
-		return UTRACE_ACTION_RESUME;
+		return UTRACE_RESUME;
 
+	__stp_tf_handler_start();
 
 	// We need the first syscall argument to see what address
 	// we're operating on.
@@ -691,7 +969,8 @@ __stp_utrace_task_finder_target_syscall_entry(struct utrace_attached_engine *eng
 			mmput(mm);
 		}
 	}
-	return UTRACE_ACTION_RESUME;
+	__stp_tf_handler_end();
+	return UTRACE_RESUME;
 }
 
 static void
@@ -711,11 +990,11 @@ __stp_target_call_vm_callback(struct stap_task_finder_target *tgt,
 	}
 
 	// Grab the path associated with this vma.
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,25)
+#ifdef STAPCONF_DPATH_PATH
+	mmpath = d_path(&(vma->vm_file->f_path), mmpath_buf, PATH_MAX);
+#else
 	mmpath = d_path(vma->vm_file->f_dentry, vma->vm_file->f_vfsmnt,
 			mmpath_buf, PATH_MAX);
-#else
-	mmpath = d_path(&(vma->vm_file->f_path), mmpath_buf, PATH_MAX);
 #endif
 	if (mmpath == NULL || IS_ERR(mmpath)) {
 		rc = -PTR_ERR(mmpath);
@@ -723,7 +1002,7 @@ __stp_target_call_vm_callback(struct stap_task_finder_target *tgt,
 			   rc, (int)tsk->pid);
 	}
 	else {
-		rc = tgt->vm_callback(tsk, 1, mmpath, vma->vm_start,
+		rc = tgt->vm_callback(tgt, tsk, 1, mmpath, vma->vm_start,
 				      vma->vm_end,
 				      (vma->vm_pgoff << PAGE_SHIFT));
 		if (rc != 0) {
@@ -734,10 +1013,18 @@ __stp_target_call_vm_callback(struct stap_task_finder_target *tgt,
 	_stp_kfree(mmpath_buf);
 }
 
+#ifdef UTRACE_ORIG_VERSION
 static u32
 __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engine,
 					     struct task_struct *tsk,
 					     struct pt_regs *regs)
+#else
+static u32
+__stp_utrace_task_finder_target_syscall_exit(enum utrace_resume_action action,
+					     struct utrace_attached_engine *engine,
+					     struct task_struct *tsk,
+					     struct pt_regs *regs)
+#endif
 {
 	struct stap_task_finder_target *tgt = engine->data;
 	unsigned long syscall_no;
@@ -747,30 +1034,34 @@ __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engi
 	struct mm_struct *mm;
 	struct vm_area_struct *vma;
 	struct __stp_tf_vma_entry *entry = NULL;
+#if defined(__ia64__)
+	struct { unsigned long *unwaddr; } _c = {.unwaddr = NULL}, *c = &_c;
+#endif
 
 	if (atomic_read(&__stp_task_finder_state) != __STP_TF_RUNNING) {
 		debug_task_finder_detach();
-		return UTRACE_ACTION_DETACH;
+		return UTRACE_DETACH;
 	}
 
 	if (tgt == NULL || tgt->vm_callback == NULL)
-		return UTRACE_ACTION_RESUME;
+		return UTRACE_RESUME;
 
 	// See if syscall is one we're interested in.
 	//
 	// FIXME: do we need to handle mremap()?
 	syscall_no = __stp_user_syscall_nr(regs);
 	if (syscall_no != MMAP_SYSCALL_NO(tsk)
+	    && syscall_no != MMAP2_SYSCALL_NO(tsk)
 	    && syscall_no != MPROTECT_SYSCALL_NO(tsk)
 	    && syscall_no != MUNMAP_SYSCALL_NO(tsk))
-		return UTRACE_ACTION_RESUME;
+		return UTRACE_RESUME;
 
 	// Get return value
 	rv_addr = __stp_user_syscall_return_value(tsk, regs);
 	if ((rc = __stp_get_user(rv, rv_addr)) != 0) {
 		_stp_error("couldn't read syscall return value for pid %d: %d",
 			   tsk->pid, rc);
-		return UTRACE_ACTION_RESUME;
+		return UTRACE_RESUME;
 	}
 
 	// We need the first syscall argument to see what address we
@@ -779,7 +1070,7 @@ __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engi
 	if ((rc = __stp_get_user(arg0, arg0_addr)) != 0) {
 		_stp_error("couldn't read syscall arg 0 for pid %d: %d",
 			   tsk->pid, rc);
-		return UTRACE_ACTION_RESUME;
+		return UTRACE_RESUME;
 	}
 
 #ifdef DEBUG_TASK_FINDER_VMA
@@ -787,11 +1078,13 @@ __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engi
 		  "tsk %d found %s(0x%lx), returned 0x%lx\n",
 		  tsk->pid,
 		  ((syscall_no == MMAP_SYSCALL_NO(tsk)) ? "mmap"
-		   : ((syscall_no == MPROTECT_SYSCALL_NO(tsk)) ? "mprotect"
-		      : ((syscall_no == MUNMAP_SYSCALL_NO(tsk)) ? "munmap"
-			 : "UNKNOWN"))),
+		   : ((syscall_no == MMAP2_SYSCALL_NO(tsk)) ? "mmap2"
+		      : ((syscall_no == MPROTECT_SYSCALL_NO(tsk)) ? "mprotect"
+			 : ((syscall_no == MUNMAP_SYSCALL_NO(tsk)) ? "munmap"
+			    : "UNKNOWN")))),
 		  arg0, rv);
 #endif
+	__stp_tf_handler_start();
 
 	// Try to find the vma info we might have saved.
 	if (arg0 != (unsigned long)NULL)
@@ -832,7 +1125,7 @@ __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engi
 				// FIXME: We'll need to figure out to
 				// retrieve the path of a deleted
 				// vma.
-				rc = tgt->vm_callback(tsk, 0, NULL,
+				rc = tgt->vm_callback(tgt, tsk, 0, NULL,
 						      entry->vm_start,
 						      entry->vm_end,
 						      (entry->vm_pgoff
@@ -866,7 +1159,7 @@ __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engi
 				// FIXME: We'll need to figure out to
 				// retrieve the path of a deleted
 				// vma.
-				rc = tgt->vm_callback(tsk, 0, NULL,
+				rc = tgt->vm_callback(tgt, tsk, 0, NULL,
 						      entry->vm_start,
 						      entry->vm_end,
 						      (entry->vm_pgoff
@@ -899,7 +1192,8 @@ __stp_utrace_task_finder_target_syscall_exit(struct utrace_attached_engine *engi
 		// Cleanup by deleting the saved vma info.
 		__stp_tf_remove_vma_entry(entry);
 	}
-	return UTRACE_ACTION_RESUME;
+	__stp_tf_handler_end();
+	return UTRACE_RESUME;
 }
 
 struct utrace_engine_ops __stp_utrace_task_finder_ops = {
@@ -915,12 +1209,15 @@ stap_start_task_finder(void)
 	struct task_struct *grp, *tsk;
 	char *mmpath_buf;
 
+	debug_task_finder_report();
 	mmpath_buf = _stp_kmalloc(PATH_MAX);
 	if (mmpath_buf == NULL) {
 		_stp_error("Unable to allocate space for path");
 		return ENOMEM;
 	}
 
+	__stp_tf_vma_initialize();
+
 	atomic_set(&__stp_task_finder_state, __STP_TF_RUNNING);
 
 	rcu_read_lock();
@@ -930,8 +1227,14 @@ stap_start_task_finder(void)
 		size_t mmpathlen;
 		struct list_head *tgt_node;
 
-		rc = stap_utrace_attach(tsk, &__stp_utrace_task_finder_ops, 0,
-					__STP_TASK_FINDER_EVENTS);
+		/* Skip over processes other than that specified with
+		 * stap -c or -x. */
+		if (_stp_target && tsk->tgid != _stp_target)
+			continue;
+
+		rc = __stp_utrace_attach(tsk, &__stp_utrace_task_finder_ops, 0,
+					 __STP_TASK_FINDER_EVENTS,
+					 UTRACE_RESUME);
 		if (rc == EPERM) {
 			/* Ignore EPERM errors, which mean this wasn't
 			 * a thread we can attach to. */
@@ -982,6 +1285,8 @@ stap_start_task_finder(void)
 			/* pid-based target */
 			else if (tgt->pid != 0 && tgt->pid != tsk->pid)
 				continue;
+			/* Notice that "pid == 0" (which means to
+			 * probe all threads) falls through. */
 
 			list_for_each(cb_node, &tgt->callback_list_head) {
 				struct stap_task_finder_target *cb_tgt;
@@ -990,32 +1295,19 @@ stap_start_task_finder(void)
 						    callback_list);
 				if (cb_tgt == NULL)
 					continue;
-					
-				// Call the callback.  Assume that if
-				// the thread is a thread group
-				// leader, it is a process.
-				if (cb_tgt->callback != NULL) {
-					rc = cb_tgt->callback(tsk, 1,
-							      (tsk->pid == tsk->tgid),
-							      cb_tgt);
-					if (rc != 0) {
-						_stp_error("attach callback for %d failed: %d",
-							   (int)tsk->pid, rc);
-						goto stf_err;
-					}
-				}
 
 				// Set up events we need for attached tasks.
-				rc = stap_utrace_attach(tsk, &cb_tgt->ops,
-							cb_tgt,
-							__STP_ATTACHED_TASK_EVENTS(cb_tgt));
+				rc = __stp_utrace_attach(tsk, &cb_tgt->ops,
+							 cb_tgt,
+							 __STP_ATTACHED_TASK_EVENTS,
+							 UTRACE_STOP);
 				if (rc != 0 && rc != EPERM)
 					goto stf_err;
 				cb_tgt->engine_attached = 1;
 			}
 		}
 	} while_each_thread(grp, tsk);
- stf_err:
+stf_err:
 	rcu_read_unlock();
 
 	_stp_kfree(mmpath_buf);
@@ -1025,10 +1317,31 @@ stap_start_task_finder(void)
 static void
 stap_stop_task_finder(void)
 {
+#ifdef DEBUG_TASK_FINDER
+	int i = 0;
+#endif
+
 	atomic_set(&__stp_task_finder_state, __STP_TF_STOPPING);
 	debug_task_finder_report();
 	stap_utrace_detach_ops(&__stp_utrace_task_finder_ops);
 	__stp_task_finder_cleanup();
 	debug_task_finder_report();
 	atomic_set(&__stp_task_finder_state, __STP_TF_STOPPED);
+
+	/* Now that all the engines are detached, make sure
+	 * all the callbacks are finished.  If they aren't, we'll
+	 * crash the kernel when the module is removed. */
+	while (atomic_read(&__stp_inuse_count) != 0) {
+		schedule();
+#ifdef DEBUG_TASK_FINDER
+		i++;
+#endif
+	}
+#ifdef DEBUG_TASK_FINDER
+	if (i > 0)
+		printk(KERN_ERR "it took %d polling loops to quit.\n", i);
+#endif
 }
+
+
+#endif /* TASK_FINDER_C */
