@@ -51,40 +51,24 @@ static void _stp_do_relocation(const char __user *buf, size_t count)
       if (strcmp (_stp_modules[mi]->name, msg.module))
         continue;
 
+      if (!strcmp (".note.gnu.build-id", msg.reloc)) {
+        _stp_modules[mi]->notes_sect = msg.address;   /* cache this particular address  */
+      }
+
       for (si=0; si<_stp_modules[mi]->num_sections; si++)
         {
-          if (strcmp (_stp_modules[mi]->sections[si].symbol, msg.reloc))
+          if (strcmp (_stp_modules[mi]->sections[si].name, msg.reloc))
             continue;
-
-          _stp_modules[mi]->sections[si].addr = msg.address;
+          else
+            {
+              _stp_modules[mi]->sections[si].addr = msg.address;
+              break;
+            }
         } /* loop over sections */
     } /* loop over modules */
 }
 
 
-static int _stp_compare_addr(const void *p1, const void *p2)
-{
-	struct _stp_symbol *s1 = (struct _stp_symbol *)p1;
-	struct _stp_symbol *s2 = (struct _stp_symbol *)p2;
-	if (s1->addr == s2->addr)
-		return 0;
-	if (s1->addr < s2->addr)
-		return -1;
-	return 1;
-}
-
-static void _stp_swap_symbol(void *x, void *y, int size)
-{
-	struct _stp_symbol *a = (struct _stp_symbol *)x;
-	struct _stp_symbol *b = (struct _stp_symbol *)y;
-	unsigned long addr = a->addr;
-	const char *symbol = a->symbol;
-	a->addr = b->addr;
-	a->symbol = b->symbol;
-	b->addr = addr;
-	b->symbol = symbol;
-}
-
 static void u32_swap(void *a, void *b, int size)
 {
 	u32 t = *(u32 *)a;
@@ -157,7 +141,8 @@ static int _stp_section_is_interesting(const char *name)
 {
 	int ret = 1;
 	if (!strncmp("__", name, 2)
-	    || !strncmp(".note", name, 5)
+	    || (!strncmp(".note", name, 5) 
+		&& strncmp(".note.gnu.build-id", name, 18))
 	    || !strncmp(".gnu", name, 4)
 	    || !strncmp(".mod", name, 4))
 		ret = 0;
