@@ -32,6 +32,7 @@ struct be_derived_probe_group;
 struct dwarf_derived_probe_group;
 struct uprobe_derived_probe_group;
 struct utrace_derived_probe_group;
+struct itrace_derived_probe_group;
 struct task_finder_derived_probe_group;
 struct timer_derived_probe_group;
 struct profile_derived_probe_group;
@@ -51,16 +52,16 @@ struct module_cache;
 struct statistic_decl
 {
   statistic_decl()
-    : type(none), 
+    : type(none),
       linear_low(0), linear_high(0), linear_step(0)
-  {}    
+  {}
   enum { none, linear, logarithmic } type;
   int64_t linear_low;
   int64_t linear_high;
   int64_t linear_step;
   bool operator==(statistic_decl const & other)
   {
-    return type == other.type 
+    return type == other.type
       && linear_low == other.linear_low
       && linear_high == other.linear_high
       && linear_step == other.linear_step;
@@ -103,6 +104,7 @@ struct systemtap_session
   bool prologue_searching;
   bool tapset_compile_coverage;
   bool need_uprobes;
+  bool load_only; // flight recorder mode
 
   // Cache data
   bool use_cache;
@@ -130,7 +132,7 @@ struct systemtap_session
   // resolved globals/functions/probes for the run as a whole
   std::vector<stapfile*> files;
   std::vector<vardecl*> globals;
-  std::vector<functiondecl*> functions;
+  std::map<std::string,functiondecl*> functions;
   std::vector<derived_probe*> probes; // see also *_probes groups below
   std::vector<embeddedcode*> embeds;
   std::map<std::string, statistic_decl> stat_decls;
@@ -146,6 +148,7 @@ struct systemtap_session
   dwarf_derived_probe_group* dwarf_derived_probes;
   uprobe_derived_probe_group* uprobe_derived_probes;
   utrace_derived_probe_group* utrace_derived_probes;
+  itrace_derived_probe_group* itrace_derived_probes;
   task_finder_derived_probe_group* task_finder_derived_probes;
   timer_derived_probe_group* timer_derived_probes;
   profile_derived_probe_group* profile_derived_probes;
@@ -176,6 +179,7 @@ struct systemtap_session
   const token* last_token;
   void print_token (std::ostream& o, const token* tok);
   void print_error (const semantic_error& e);
+  void print_error_source (std::ostream&, std::string&, const token* tok);
   void print_warning (const std::string& w, const token* tok = 0);
 
   // reNB: new POD members likely need to be explicitly cleared in the ctor.
