@@ -287,6 +287,7 @@ struct print_format: public expression
       conv_unsigned_uppercase_hex,
       conv_unsigned_lowercase_hex,
       conv_string,
+      conv_char,
       conv_memory,
       conv_literal,
       conv_binary,
@@ -346,7 +347,7 @@ struct print_format: public expression
 
   static std::string components_to_string(std::vector<format_component> const & components);
   static std::vector<format_component> string_to_components(std::string const & str);
-  static bool parse_print(const std::string &name, bool &stream, 
+  static bool parse_print(const std::string &name, bool &stream,
 	            bool &format, bool &delim, bool &newline, bool &_char);
 
   void print (std::ostream& o) const;
@@ -470,6 +471,8 @@ struct block: public statement
   std::vector<statement*> statements;
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  block () {}
+  block (statement* car, statement* cdr);
 };
 
 
@@ -572,8 +575,10 @@ struct stapfile
   std::vector<functiondecl*> functions;
   std::vector<vardecl*> globals;
   std::vector<embeddedcode*> embeds;
+  std::string file_contents;
   bool privileged;
-  stapfile (): privileged (false) {}
+  stapfile (): file_contents (""),
+    privileged (false) {}
   void print (std::ostream& o) const;
 };
 
@@ -605,7 +610,7 @@ std::ostream& operator << (std::ostream& o, const probe_point& k);
 struct probe
 {
   std::vector<probe_point*> locations;
-  block* body;
+  statement* body;
   const token* tok;
   std::vector<vardecl*> locals;
   std::vector<vardecl*> unused_locals;
