@@ -33,6 +33,7 @@ public:
 
   std::ostream& newline (int indent = 0);
   void indent (int indent = 0);
+  void assert_0_indent () { assert (tablevel == 0); }
   std::ostream& line();
 
   std::ostream::pos_type tellp() { return o.tellp(); }
@@ -64,9 +65,13 @@ struct unparser
   //   ...
   // } context [MAXCONCURRENCY];
 
+  // struct {
   virtual void emit_global (vardecl* v) = 0;
-  // static TYPE global_NAME;
-  // static rwlock_t global_NAME_lock;
+  // TYPE s_NAME;  // NAME is prefixed with "s_" to avoid kernel id collisions
+  // rwlock_t s_NAME_lock;
+  // } global = {
+  virtual void emit_global_init (vardecl* v) = 0;
+  // };
 
   virtual void emit_global_param (vardecl* v) = 0;
   // module_param_... -- at end of file
